SUBROUTINE SU0YOMB(YDGEOMETRY,YDVARIABLES,YDFIELDS,YDMTRAJ,YDVARBC)

!**** *SU0YOMB*  - INITIALIZE LEVEL 0 COMMONS AND SOME HIGHER (PART 2)

!     PURPOSE.
!     --------
!           INITIALIZE LEVEL 0 COMMONS (CONSTANT ALONG ALL THE JOB)
!        AND SOME HIGHER LEVEL COMMONS. CALLS ROUTINES THAT PERFORM
!        ALL THE PREPARATIONS NEEDED TO EXECUTE MODEL. THE TASK OF
!        INITIALIZING THE COMMONS IS DIVIDED BETWEEN TWO ROUTINES
!        (SU0YOMA AND SU0YOMB) TO AVOID PROBLEMS WITH USING POINTER
!        ARRAYS WHOSE DIMENSIONS ARE NOT DEFINED UNTIL AFTER CALLING SUDIM.

!        ky: some constraints to be known for future code reorganisation (this list is not comprehensive):
!         - SUPPVI does not depend on geometry and could be called earlier in the set-up (somewhere in SU0YOMA)
!         - SUPP must be called after SUGFL and partly uses horizontal geometry.
!         - SUIOS does not depend on geometry and could be called earlier in the set-up (somewhere in SU0YOMA)
!         - SUPTRTC does not depend on geometry.
!         - SUDFI could probably be called earlier; requires to know TSTEP.
!         - SUSPNG could be called inside SUDYN, like calls to SU(E)HDF and SURAYFRIC (diffusive processes).
!         - SUVAREPS must be called before SUGRIB.

!     INTERFACE.
!     ----------
!        *CALL* *SU0YOMB*

!        EXPLICIT ARGUMENTS
!        --------------------
!        NONE

!        IMPLICIT ARGUMENTS
!        --------------------
!        NONE

!     METHOD.
!     -------
!        SEE DOCUMENTATION

!     EXTERNALS.
!     ----------
!      see below

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!        MATS HAMRUD AND PHILIPPE COURTIER  *ECMWF*
!        ORIGINAL      : 87-10-15

!     MODIFICATIONS.
!     --------------
!        C. Fischer    : 03-09-30 Remove call to suellr
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!        E .Holm       : 03-12-08  Include humidity change of variable
!                                  coefficient setup (SUJBCHVAR)
!        M.Fisher      17-03-2004  Added calls to SUJBWAVET0 and SUJBWAVTRANS
!        M.Leutbecher  07-05-2004  Added calls to SU_SUBSPACE
!        C. Fischer    : 04-02-27  Merge Aladin into Arpege/IFS su0yomb:
!                        - moved up call to susc2a (susc2 split for LAM)
!                        - all sue* routines now in the IFS setup
!                        - historical header of the file aladinized
!        V.Guidard     : 04-04-01  Add Jk computation for ALADIN
!        K. Yessad     : 04-07-06  Merge Aladin SUEIOS into Arpege/IFS SUIOS.
!        P. Termonia   : 04-08-31  - add call to SUMCUF
!        R. Buizza     : 09-03-05 Add call to SUVAREPS to initialize VAREPS
!        C. Temperton  : 06-02-07  SUALCTV avoided for shallow water model
!        S. Malardel   : 06-06-06 Add call to SULSFORC
!        R. El Khatib  : 31-Aug-2007 Initialise uninitialised variable
!        B. Strajnar   : 06-02-08 Allow call to SUINFCE for LSPFCE=.F.
!        B. Chapnik    : 15-06-08 Add setup of Jk modulation wrt/ level and parameter
!        R. El Khatib  : 24-Oct-2008 Merge sueoph and suoph
!        K. Yessad     : 09-09-08 No recalc of Gaussian lat and weights in SUDIL
!        K. Yessad     : 15-09-08 Prune conf 951.
!        A. Deckmyn    : 01-10-08 LAM wavelets
!        P. Termonia   : 22-12-08 move the call to SUDFI here (from SUINI)
!        A. Dethof     : 17-11-08 LOG NOX JB
!        K. Yessad (Aug 2009): add call to SUPPVI.
!        K. Yessad (Aug 2009): prune conf 912, externalise conf 911.
!        K.Yessad (Feb 2010): use YM_RADTC and RFORADTC
!        R. El Khatib 13-Jul-2010 Move surfex allocations below suphmse
!        R. El Khatib 13-Jul-2010 Move surfex allocations below suphmse
!        K. Yessad (Sep 2010): organigramme simplification.
!        K. Yessad (Jan 2011): new architecture for LBC modules and set-up.
!        P. Marguinaud 01-Jan-2011 IO server setup (send parameters with MPI)
!        G. Kerdraon   : Feb 2011 Call SU_GRIB_API if NOT LELAM
!        K. Yessad (jul 2011): reorder calculations in order to use new structures in LAM models.
!        T.Wilhelmsson (Aug 2011) SUSC2B => SUSC2B + SUSC2C
!        M. Fisher   7-March-2012 Move Jb setup here (from SU0YOMA)
!        K. Yessad (dec 2011): various contributions.
!        R. El Khatib 09-Mar-2012 : Unconditional call to sualdyn_ddh
!        R. El Khatib 23-Mar-2012 : Fix bounds checking issue
!        B. Bochenek (Apr 2012): call to SUNDDH moved from SU0YOMA
!        R. El Khatib 26-Jul-2012 : SUVV1 (previously part of SUVERT)
!                                 + SUFPG (previously part of SUBFPOS)
!        M.Fisher 15-Feb-2013 Introduce CVA_STRUCT, SCALP_STRUCT, CVA_DATA
!        K. Yessad (july 2013): various modifications for geometry set-up.
!        T. Wilhelmsson and K. Yessad (Oct 2013) Geometry and setup refactoring.
!        D. Degrauwe, 2013-11 : Setup of flexible interface (INTFLEX).
!        R. El Khatib  04-Jul-2014 call sualspa1 before sualspa to facilitate the 
!         subsequent allocation of the spectral structure by alloc_spec
!        K. Yessad (july 2014): some reorganisation in set-up.
!      R. El Khatib 04-Aug-2014 Pruning of the conf. 927/928
!      P. Marguinaud 10-Oct-2014 Add LGRIB_API
!      R. El Khatib : 03-Dec-2014 skeleton of the configuration 903
!      A. Geer      27-Jul-2015   VarBC is now an object, for OOPS
!      R. El Khatib 17-Aug-2016 move suoph up to su0yoma
!     ------------------------------------------------------------------

USE GEOMETRY_MOD , ONLY : GEOMETRY
USE VARIABLES_MOD, ONLY : VARIABLES
USE FIELDS_MOD , ONLY : FIELDS
USE MTRAJ_MOD  , ONLY : MTRAJ
USE PARKIND1 , ONLY : JPIM, JPRB
USE YOMHOOK  , ONLY : LHOOK, DR_HOOK
USE YOMDIMF  , ONLY : YRDIMF
USE YOMLUN   , ONLY : NULOUT, NULBDESH
USE YOMCT0   , ONLY : LR2D ,NCONF, LBACKG, LSPBSBAL, LELAM,&
 & LRETCFOU, LWRTCFOU, LOBS, LOBSC1, LSCREEN, LIFSTRAJ, LIFSMIN, NFPOS,&
 & NFRCO, LGRIB_API
USE YOMLDDH  , ONLY : YRLDDH
USE YOMVAR   , ONLY : LMODERR , LVARBC, LJCDFI, LMONITOR_OBSTATFC,&
 & LBACKGERENORM, LCVALPHA
USE YOMJG    , ONLY : JB_STRUCT
USE YOMJQ    , ONLY : LSTATMERR
USE YOMSIMPHL, ONLY : YRSIMPHL
USE YOMLCZ   , ONLY : L_SUBSPACE_SVS, LFORCEWR
USE YOMCVA   , ONLY : CVA_STRUCT, SCALP_STRUCT, CVA_DATA
USE TRAJECTORY_MOD, ONLY : LTRAJRESET, LTRAJHR, ALLOCATE_TRAJECTORY,&
 & READ_TRAJECTORY, GET_TRAJ_GRID, LREADGPTRAJ, LTRAJHR_ALTI
USE YOM_YGFL , ONLY : YGFL
USE YOMGWDIAG, ONLY : SETUP_GWDIAG
USE YEMJK    , ONLY : LEJK
USE YOMCOAPHY, ONLY : YRCOAPHY
USE YOMINI   , ONLY : LDFI
USE STOPH_MIX, ONLY : YRSTOPH
USE YOMSPSDT,  ONLY : LSPSDT
USE YOE_CUCONVCA, ONLY : INI_CUCONVCA
USE MODULE_RADTC_MIX, ONLY : YM_RADTC ,SUPTRTC
USE YOMRCOEF , ONLY : YRRCOEF
USE ELBC0B_MOD,ONLY : SUELBC0B
USE ELBC0C_MOD,ONLY : SUELBC0C
USE YOMIO_SERV,ONLY : IO_SERV_C001
USE YOMFP_SERV,ONLY : FP_SERV_C001
USE YOM_GRIB_CODES, ONLY : NGRBNOXLOG
USE YOMGRIB  , ONLY : NGRBSP3, NGRBSP2
USE SPNG_MOD  ,ONLY : SUSPNG
USE YEMENSHYB, ONLY : SETUP_HYBENS, ALLOCATE_ENS
USE INTDYN_MOD, ONLY : SUINTDYN
USE CONTROL_VECTORS, ONLY : CTLVEC_STRUCT, CONTROL_VECTOR
USE YOMTRAJ  , ONLY : LTRAJALLOC, TRAJEC
USE YOMMP0   , ONLY : NPROC, NPRTRV
USE YOMSP,     ONLY : SPA3
USE SPECTRAL_FIELDS_MOD
USE TESTVAR_MIX,ONLY: SETUP_TESTVAR
USE VARBC_CLASS, ONLY : CLASS_VARBC
USE YOMSPJB  , ONLY : BACKGROUND, ALLOCATE_JB_REF_STATE
USE MPL_MODULE, ONLY : MPL_END, MPL_BARRIER
USE IOSTREAM_MIX,   ONLY : IOSTREAM_STATS
USE YOMRIP, ONLY : YRRIP
USE SPP_MOD  , ONLY : YSPP_CONFIG, YSPP
USE YOMPHY,   ONLY : YRPHY

!     ------------------------------------------------------------------

IMPLICIT NONE

TYPE(GEOMETRY),    INTENT(INOUT) :: YDGEOMETRY
TYPE(VARIABLES),   INTENT(IN)    :: YDVARIABLES
TYPE(FIELDS),      INTENT(INOUT) :: YDFIELDS
TYPE(MTRAJ),       INTENT(INOUT) :: YDMTRAJ
TYPE(CLASS_VARBC), INTENT(INOUT) :: YDVARBC

CHARACTER (LEN = 35) ::  CLINE
CHARACTER(LEN=40) :: CLFILE
CHARACTER (LEN=3)  :: CLMAX

INTEGER(KIND=JPIM) :: ICONF, IFACTD, INIMES

LOGICAL :: LLIMST, LLDIMO, LL_ALLOC_RLANBUF

INTEGER(KIND=JPIM) :: JGFL, ISTEP, JSTGLO, ICEND, IBL, ISIZEG
INTEGER(KIND=JPIM) :: IGRIB(YRDIMF%NS3D+YRDIMF%NS2D)
INTEGER(KIND=JPIM) :: ISURF(NPRTRV)
LOGICAL :: LLASTRAJ
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5(:,:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGMV5S(:,:,:)
REAL(KIND=JPRB), ALLOCATABLE :: ZGFL5(:,:,:,:)

TYPE(CONTROL_VECTOR) :: YLTEMP

REAL(KIND=JPRB) :: ZHOOK_HANDLE
!     ------------------------------------------------------------------

#include "gstats_output_ifs.intfb.h"
#include "get_spp_conf.intfb.h"
#include "ini_spp.intfb.h"
#include "inifger.intfb.h"
#include "sualcan.intfb.h"
#include "sualcos.intfb.h"
#include "sualcosjo.intfb.h"
#include "sualctv.intfb.h"
#include "sualdyn_ddh.intfb.h"
#include "sualfoutc.intfb.h"
#include "sualgco.intfb.h"
#include "sualges.intfb.h"
#include "suallt.intfb.h"
#include "sualmdh.intfb.h"
#include "suspvariables.intfb.h"
#include "sualspa.intfb.h"
#include "sualnud.intfb.h"
#include "sualspajb.intfb.h"
#include "suanebuf.intfb.h"
#include "sualtdh.intfb.h"
#include "sufpg.intfb.h"
#include "subfpos.intfb.h"
#include "sucfu.intfb.h"
#include "sudfi.intfb.h"
#include "sudimo.intfb.h"
#include "sudyn.intfb.h"
#include "suecges.intfb.h"
#include "suejbbal.intfb.h"
#include "suejbcov.intfb.h"
#include "suejk.intfb.h"
#include "suejknorm.intfb.h"
#include "suelges.intfb.h"
#include "suelljk.intfb.h"
#include "suemodjk.intfb.h"
#include "suescal.intfb.h"
#include "suevargp.intfb.h"
#include "sugrib.intfb.h"
#include "su_grib_api.intfb.h"
#include "sufdb.intfb.h"
#include "suiau.intfb.h"
#include "suinfce.intfb.h"
#include "sufw.intfb.h"
#include "suios.intfb.h"
#include "sujb.intfb.h"
#include "sujbbal.intfb.h"
#include "sujbcov.intfb.h"
#include "sujbtest.intfb.h"
#include "sujbwavelet0.intfb.h"
#include "sujbwavelet.intfb.h"
#include "sujbwavelet_stdevs.intfb.h"
#include "sujbwavrenorm.intfb.h"
#include "sujbwavstats.intfb.h"
#include "sujbwavtrans.intfb.h"
#include "sujq.intfb.h"
#include "sujbchvar.intfb.h"
#include "sulcz.intfb.h"
#include "sulfi.intfb.h"
#include "sulsforc.intfb.h"
#include "sumcclag.intfb.h"
#include "sumddh.intfb.h"
#include "sumoderr.intfb.h"
#include "sunddh.intfb.h"
#include "suphy.intfb.h"
#include "supp.intfb.h"
#include "suppvi.intfb.h"
#include "surand1.intfb.h"
#include "surcoftc.intfb.h"
#include "sures.intfb.h"
#include "surinc.intfb.h"
#include "surlx.intfb.h"
#include "susc2b.intfb.h"
#include "susc2c.intfb.h"
#include "suscal.intfb.h"
#include "susimpr.intfb.h"
#include "suspsdt.intfb.h"
#include "suvareps.intfb.h"
#include "suxfu.intfb.h"
#include "su_subspace.intfb.h"
#include "sumcuf.intfb.h"
#include "sucoaphy.intfb.h"
#include "suoaf.intfb.h"
#include "sualobs.intfb.h"
#include "suejbwavelet.intfb.h"
#include "suejbwavelet_bmatrix.intfb.h"
#include "io_serv_suiosctmpl.intfb.h"
#include "fp_serv_suiosctmpl.intfb.h"
#include "fp_serv_cpfpfilter.intfb.h"
#include "allocate_empty_trajectory.intfb.h"
#include "suintflex.intfb.h"
#include "sujacos.intfb.h"
#include "suja.intfb.h"
#include "sualpfld.intfb.h"
#include "get_hybrid_ens.intfb.h"
#include "suwfp.intfb.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('SU0YOMB',0,ZHOOK_HANDLE)
ASSOCIATE(YDGFL5=>YDMTRAJ%YRGFL5,YDGMV5=>YDMTRAJ%YRGMV5, YDGFL=>YDFIELDS%YRGFL,YDGMV=>YDFIELDS%YRGMV,  &
 & YDSURF=>YDFIELDS%YRSURF, YDDIM=>YDGEOMETRY%YRDIM, YDDIMV=>YDGEOMETRY%YRDIMV, YDGEM=>YDGEOMETRY%YRGEM,  &
 & YDMP=>YDGEOMETRY%YRMP, YDLAP=>YDGEOMETRY%YRLAP, YDSTA=>YDGEOMETRY%YRSTA, YDVAB=>YDGEOMETRY%YRVAB,  &
 & YDVETA=>YDGEOMETRY%YRVETA, YDVFE=>YDGEOMETRY%YRVFE, YDSTADLR=>YDGEOMETRY%YRSTADLR, &
 & YDCSGLEG=>YDGEOMETRY%YRCSGLEG, YDVSPLIP=>YDGEOMETRY%YRVSPLIP, YDVSLETA=>YDGEOMETRY%YRVSLETA,  &
 & YDHSLMER=>YDGEOMETRY%YRHSLMER, &
 & YDCSGEOM=>YDGEOMETRY%YRCSGEOM, YDCSGEOM_NB=>YDGEOMETRY%YRCSGEOM_NB, &
 & YDGSGEOM=>YDGEOMETRY%YRGSGEOM, YDGSGEOM_NB=>YDGEOMETRY%YRGSGEOM_NB, YDSPGEOM=>YDGEOMETRY%YSPGEOM)

ASSOCIATE(NCHEM_ASSIM=>YGFL%NCHEM_ASSIM, NDIM5=>YGFL%NDIM5, YCOMP=>YGFL%YCOMP, &
 & NPHYINT=>YRCOAPHY%NPHYINT, &
 & NCMAX=>YDDIM%NCMAX, NGPBLKS=>YDDIM%NGPBLKS, NMSMAX=>YDDIM%NMSMAX, &
 & NPROMA=>YDDIM%NPROMA, NRESOL=>YDDIM%NRESOL, NSMAX=>YDDIM%NSMAX, &
 & NUMP=>YDDIM%NUMP, &
 & NS2D=>YRDIMF%NS2D, NS3D=>YRDIMF%NS3D, &
 & NFLEVG=>YDDIMV%NFLEVG, NFLEVL=>YDDIMV%NFLEVL, &
 & NGPTOT=>YDGEM%NGPTOT, RSTRET=>YDGEM%RSTRET, &
 & YT5=>YDGMV5%YT5, &
 & MYMS=>YDLAP%MYMS, &
 & LSDDH=>YRLDDH%LSDDH, &
 & NALLMS=>YDMP%NALLMS, NPTRLL=>YDMP%NPTRLL, NPTRMS=>YDMP%NPTRMS, &
 & NUMLL=>YDMP%NUMLL, &
 & LRCOEF=>YRRCOEF%LRCOEF, &
 & LSTOPH=>YRSTOPH%LSTOPH, &
 & LRAYSP=>YRSIMPHL%LRAYSP, &
 & LWINDFARM=>YRPHY%LWINDFARM)


!     ------------------------------------------------------------------
CALL GSTATS(39,0)
CLINE='----------------------------------'

ICONF=NCONF/100

!*    Initialize vertical interpolator
WRITE(NULOUT,*) '---- Set up vertical interpolator ---',CLINE
CALL SUPPVI

!*    Initialize post-processing
WRITE(NULOUT,*) '---- Set up post-processing ---------',CLINE
CALL SUPP(' ')

!*    Initialize I/O-scheme
WRITE(NULOUT,*) '---- Set up I/O scheme --------------',CLINE
CALL SUIOS

!*    Full Post-processing (horizontal part) after model geometry and before spectral transforms
IF (NFPOS /= 0) THEN
  WRITE(NULOUT,*) '- Set up F-post processing, Horizontal geometry -',CLINE
  CALL SUFPG(YDGEOMETRY)
ENDIF

!*    Initialize LFI files parameters
WRITE(NULOUT,*) '---- Set up lfi files ----',CLINE
CALL SULFI(NULOUT,NSMAX,NCMAX,RSTRET,LLIMST,INIMES,IFACTD)

!*    Initialize Aladin Jk handling
IF (LELAM.AND.(ICONF == 1)) THEN
  WRITE(NULOUT,*) '---- Set up Aladin Jk handling -',CLINE
  CALL SUEJK
ELSE
  LEJK=.FALSE.
ENDIF

!*    Initialize DDH (Horizontal domains diagnostics)
WRITE(NULOUT,*) '------ Set up DDH diagnostics --------',CLINE
CALL SUNDDH(YDGEOMETRY,YDFIELDS%YRSURF)

IF (LBACKG .OR. LOBSC1 .OR. NCONF == 401 .OR. NCONF == 501 .OR. NCONF == 601 .OR. NCONF == 801 .OR. NCONF == 701) THEN

  IF (.NOT.LR2D) THEN
    WRITE(NULOUT,*) '------ Set up Jb parameters ------------',CLINE
    ALLOCATE(JB_STRUCT)
    ALLOCATE(CVA_DATA)
    CALL SUJB(YDGEOMETRY,JB_STRUCT, CVA_DATA)
  ENDIF
ENDIF

WRITE(NULOUT,*) ' LIFSMIN = ', LIFSMIN
IF(ASSOCIATED(JB_STRUCT)) THEN
  LLDIMO=.NOT. JB_STRUCT%WJBCONF%LJBWSTATS .OR. .NOT. LIFSMIN
  WRITE(NULOUT,*) ' JB_STRUCT%WJBCONF%LJBWAVSTATS = ', JB_STRUCT%WJBCONF%LJBWSTATS
ELSE
  LLDIMO=.NOT. LIFSMIN
ENDIF

IF (LLDIMO) THEN ! OBSERVATION SETUP CAN BE SKIPPED WHEN COMPUTING JB
  WRITE(NULOUT,*) '---- Initialize dimensions for obs. processing...'

  !     Initialize dimensions for obs. processing, Jo and Jg related arrays
  CALL SUDIMO(YDGEOMETRY,NULOUT)

  !     Setup observation array format
  IF (LOBS) THEN
    WRITE(NULOUT,*) '------ Set up observation format ---',CLINE
    CALL SUOAF
    CALL SUALOBS ! was originally in ./ifs/setup/suallo.F90
  ENDIF

  !     Set up for variational bias correction. For now, do not allow
  !     any other parameters to be part of the control vector.
  IF(ASSOCIATED(CVA_DATA)) CVA_DATA%NVAPARAM=0
  IF (LOBS .AND. LVARBC) THEN
    IF (IABS(NCONF/100)==1) THEN
      IF (.NOT.ASSOCIATED(CVA_DATA)) CALL ABOR1('CVA_DATA has not been set up')
      CALL YDVARBC%SETUP_MIN(CVA_DATA%NVAPARAM)
      IF (CVA_DATA%NVAPARAM==0) LVARBC = .FALSE.
    ENDIF
  ENDIF
ENDIF

 IF (ASSOCIATED(CVA_DATA)) CVA_DATA%NALPHA=0
 IF (LCVALPHA) THEN
  IF (IABS(NCONF/100)==1) THEN 
   IF (.NOT.ASSOCIATED(CVA_DATA)) CALL ABOR1('CVA_DATA has not been set up')
   CALL SUJA(CVA_DATA)
   IF (CVA_DATA%NALPHA==0) LCVALPHA=.FALSE.
  ENDIF 
 ENDIF

!*     Allocate spectral arrays
WRITE(NULOUT,*) '-- Set up spectral arrays allocation-',CLINE
CALL SUSPVARIABLES
IF(NCONF /= 901.AND.NCONF /= 923) THEN
  ISURF(:) = 1
  IGRIB(1:NS3D)=NGRBSP3(:)
  IGRIB(NS3D+1:NS3D+NS2D)=NGRBSP2(:)
  WRITE(NULOUT,*)'SU0YOMB: calling ALLOCATE_SPEC, ISURF = ',ISURF(:)
  CALL ALLOCATE_SPEC(SPA3, NFLEVL, NFLEVG, NUMP, MYMS, NSMAX, NMSMAX, NALLMS,&
                   & NPTRMS, NUMLL, NPTRLL, ISURF, NS3D, NS2D, IGRIB)
  CALL SUALSPA(YDGEOMETRY)
ELSE
  WRITE(NULOUT,*) 'SU0YOMB: No call for SUALSPA1'
ENDIF

!*    Allocate nudging arrays
WRITE(NULOUT,*) '-- Set up nudging arrays allocation-',CLINE
CALL SUALNUD(YDGEOMETRY)

IF ( NCONF == 701 ) THEN
  CALL SUALCOSJO
  CALL SUALGES(YDGEOMETRY,JB_STRUCT)
  IF (LELAM) CALL SUELGES(YDGEOMETRY)
ENDIF

!*    Initialize Jb

SETUP_JB: IF (LBACKG .OR. LOBSC1 .OR. NCONF == 401 .OR. NCONF == 501 .OR. NCONF == 601 .OR. NCONF == 801) THEN

!     Setup cost function arrays
  WRITE(NULOUT,*) '------ Set up cost functions -----------',CLINE
  CALL SUALCOS(YDGEOMETRY)

!    Allocate Jb spectral arrays

  WRITE(NULOUT,*) '------ Allocate Jb spectral arrays -----',CLINE
  IF (NCONF==131 .OR. NCONF/100==4 .OR. NCONF/100==5 .OR. NCONF/100==6 .OR. NCONF/100==8 .OR. &
    & (LTRAJHR .AND. LIFSTRAJ)) THEN
    ALLOCATE(BACKGROUND)
  ENDIF
  CALL SUALSPAJB(YDGEOMETRY,JB_STRUCT%JB_DATA%SPJB,JB_STRUCT)

!     Setup dimensioning for wavelet Jb
  WRITE(NULOUT,*) '------ Setup Wavelet Jb dimensioning ---',CLINE
IF (ASSOCIATED(JB_STRUCT)) THEN
  IF (JB_STRUCT%WJBCONF%LJBWAVELET) THEN
    IF (LELAM) THEN
      WRITE(NULOUT,*) '-- Setup Aladin wavelet Jb: preliminaries'
      WRITE(NULOUT,*) '(I have to do this before getting to the Control Vector setup!)'
      CALL SUEJBWAVELET(YDDIMV,JB_STRUCT)
      JB_STRUCT%WJBCONF%N_WAVELET_SCALES=0 ! We don't use it, so for safety, set it to 0
    ELSE
      WRITE(NULOUT,*) '-- Setup spectral filters for wavelet Jb'
      CALL SUJBWAVELET0(YDDIM,JB_STRUCT)
      WRITE(NULOUT,*) '-- Define multiple-resolution transforms for wavelet Jb'
      CALL SUJBWAVTRANS(YDDIM,JB_STRUCT)
    ENDIF
  ELSE
    JB_STRUCT%WJBCONF%N_WAVELET_SCALES=0
  ENDIF
ENDIF

  WRITE(NULOUT,*) '------- Allocate Jb arrays ----------',CLINE
  CALL SUALGES(YDGEOMETRY,JB_STRUCT)
  IF (LELAM) CALL SUELGES(YDGEOMETRY)

!*    Initialize gridpoint buffers for analysis errors
  IF (NCONF /= 901) THEN
    WRITE(NULOUT,*) '---- Set up gridpoint buffers for analysis errors ---',CLINE
    CALL SUANEBUF(YDGEOMETRY,JB_STRUCT)
  ENDIF
ENDIF SETUP_JB

!     Define pointers for transmission coefficients (simp. radiation).
WRITE(NULOUT,*) '-- Set up pointers for transmission coefficients-',CLINE
IF (LRCOEF.OR.LRETCFOU.OR.LWRTCFOU) THEN
  CALL SUPTRTC(.TRUE.,YM_RADTC)
ELSE
  CALL SUPTRTC(.FALSE.,YM_RADTC)
ENDIF

!     Allocate Fourier arrays containing transmission coefficients.
IF (LRETCFOU.OR.LWRTCFOU) THEN
  WRITE(NULOUT,*) '-- Set up Fourier arrays allocation-',CLINE
  CALL SUALFOUTC(YDGEOMETRY)
ENDIF

!*    Initialize geometry parameters for gridpoint
!     error standard deviations in Aladin C+I zone (YEMVARGP)
IF (LELAM) THEN
  WRITE(NULOUT,*) '---- Set up Aladin geometry: YEMVARGP -',CLINE
  IF (ICONF == 1) CALL SUEVARGP(YDGEOMETRY)
ENDIF

!*    Set up DFI (we need geometry for SSDFI)
IF (LDFI) THEN
  WRITE(NULOUT,*) '---- Set up DFI: SUDFI -----',CLINE
  CALL SUDFI
ENDIF

!*    Allocate OI CANARI grid points arrays
IF(ICONF == 7) THEN
  WRITE(NULOUT,*) '------- Allocate CANARI arrays ----',CLINE
  CALL SUALCAN(YDGEOMETRY)
ENDIF

!*    Initialize basis of Fourier coefficients for 'TC' coefficients.
IF (LRETCFOU.OR.LWRTCFOU) THEN
  WRITE(NULOUT,*) '---- Set up basis of Fourier for TC -',CLINE
  CALL SURCOFTC(YDGEOMETRY)
ENDIF

!*    Allocate YOMGCO arrays.
IF(NFRCO /= 0) THEN
  WRITE(NULOUT,*) '---- Allocate YOMGCO arrays --------',CLINE
  CALL SUALGCO(YDGEM)
ENDIF

!*    Setup TESTVAR.
WRITE(NULOUT,*) '---- Set up TESTVAR ----------',CLINE
CALL SETUP_TESTVAR

!     Initialize model:

!*    Initialize Dynamics
WRITE(NULOUT,*) '---- Set up model dynamics ----------',CLINE
CALL SUDYN(YDGEOMETRY,NULOUT)

!*    Initialize Relaxation
WRITE(NULOUT,*) '------ Set up Relaxation ',CLINE
CALL SURLX(YDDIMV,NULOUT)

!*    Initialize new sponge
WRITE(NULOUT,*) '---- Set up new sponge ----------',CLINE
CALL SUSPNG(YDDIMV%NFLEVG, YDSTA%STZ)

!*    Initialize control of DFI: SUFW
IF (LDFI .OR. LJCDFI ) THEN
  WRITE(NULOUT,*) '---- Set up DFI initialization: SUFW ',CLINE
  CALL SUFW(YDLAP,YDDIM,YDGEOMETRY%YREGEO,YDGEOMETRY%YRELAP)
ENDIF

!*    Initialize Large Scale Forcings
WRITE(NULOUT,*) '---- Set up large scale forcings ----------',CLINE
CALL SULSFORC(NULOUT)

IF(NPHYINT > 0)CALL SUCOAPHY(YDGEOMETRY)


IF (.NOT.LR2D) THEN
  !*    Initialize Physics
  WRITE(NULOUT,*) '---- Set up model physics -----------',CLINE
  CALL SUPHY(YDGEOMETRY,NULOUT)
  !*    Initialize special keys for the climate version 2nd part
  WRITE(NULOUT,*) '---- Set up MCC climate model keys (lagged part) --',CLINE
  CALL SUMCCLAG(YDGEM,NULOUT)
ENDIF

!*    Initialize cumulated fluxes requests
WRITE(NULOUT,*) '------ Set up cumulated fluxes diags ---',CLINE
CALL SUCFU(YDGEOMETRY,NULOUT)

!*    Initialize instantaneous fluxes requests
WRITE(NULOUT,*) '------ Set up instantaneous fluxes diags ',CLINE
CALL SUXFU(YDGEOMETRY,NULOUT)

!*    Initialize variables for VAREPS (NB: must be called before SUGRIB)
WRITE(NULOUT,*) '- Set up VAREPS configuration',CLINE
CALL SUVAREPS

!*    Initialize filter for monitoring the coupling updates
WRITE(NULOUT,*) '------ Set up monitoring coupling-updates',CLINE
CALL SUMCUF(YDDIM)

!*    Initialise restart mechanism
WRITE(NULOUT,*) '---- Set up restart mechanism ------',CLINE
CALL SURES(NULOUT)

!*    Initialize buffers for gridpoint scanning, part B
IF (NCONF /= 901 .AND. NCONF /= 903) THEN
  WRITE(NULOUT,*) '---- Set up gridpoint scanning, part B ----',CLINE
  CALL SUSC2B(YDGEOMETRY)
ENDIF

!*    Initialize buffers for gridpoint scanning, part C
IF (NCONF /= 901) THEN
  WRITE(NULOUT,*) '---- Set up gridpoint scanning, part C ----',CLINE
  CALL SUSC2C(YDGEOMETRY,YDVARIABLES,YDFIELDS%YRGFL,YDFIELDS%YRGMV,YDFIELDS%YRSURF)
ENDIF

!*    Initialize forcing by coarser model: part C
IF (LELAM) THEN
  WRITE(NULOUT,*) '--- Set up forcing by coarser model part C ---------',CLINE
  CALL SUELBC0C(YDGEOMETRY,YDFIELDS%YRGMV)
ENDIF

!*    Initialize forcing by coarser model: part B
IF (LELAM) THEN
  WRITE(NULOUT,*) '--- Set up forcing by coarser model part B ---------',CLINE
  CALL SUELBC0B(YDGEOMETRY,YDFIELDS%YRGMV)
ENDIF

!*    Initialize IAU handling
WRITE(NULOUT,*) '---- Set up IAU handling -',CLINE
CALL SUIAU

!*    Initialize domains and masks for DDH

CALL SUALMDH(YDGEM)

IF ( LSDDH ) THEN
  WRITE(NULOUT,*) '---- Set up DDH diagnostic domains ',CLINE
  CALL SUMDDH(YDGEOMETRY)
ENDIF

IF (.NOT.LSDDH.AND.LRAYSP) THEN
  WRITE(NULOUT,*) '---- Set up for simp.rad.if not ddh',CLINE
  CALL SUSIMPR(YDGEOMETRY)
ENDIF

!*    Initialize GRIB coding parameters
WRITE(NULOUT,*) '---- Set up files : GRIB parameters -',CLINE
CALL SUGRIB(YDDIM)

!*    Full Post-processing (2nd part)
IF (NFPOS /= 0) THEN
  WRITE(NULOUT,*) '- Set up F-post processing, bundled part',CLINE
  CALL SUBFPOS(YDGEOMETRY,KMDLRESOL=NRESOL)
ENDIF

!*    GRIB API
IF (LGRIB_API) THEN
  WRITE(NULOUT,*) '- Set up GRIB API usage',CLINE
  CALL SU_GRIB_API(YDGEOMETRY)
ELSE
  WRITE(NULOUT,*) 'Call to SU_GRIB_API switched off',CLINE
ENDIF

! FDB
CALL SUFDB

!*    Memory allocation for cumulated DDH arrays (horizontal domains diags)
WRITE(NULOUT,*) '---- Set up DDH diags allocation --',CLINE
CALL SUALTDH(YDDIMV)

!*    Memory allocation for dynamical DDH tendencies arrays
WRITE(NULOUT,*) '---- Set up dynamical DDH arrays allocation --',CLINE
CALL SUALDYN_DDH(YDGEOMETRY)

!     Setup model error arrays
IF (LMODERR.OR.LSTATMERR) THEN
  WRITE(NULOUT,*) '-- Setup model error arrays-',CLINE
  CALL SUMODERR(YDGEOMETRY,YDFIELDS%YRGMV)
ENDIF

!     Allocate space for control variable
IF (NCONF/100 /= 9 .AND. NCONF /= 1 .AND. NCONF /= 302 .AND. NCONF /= 201 .AND. NCONF /= 701) THEN
  IF(ASSOCIATED(JB_STRUCT)) THEN
    WRITE(NULOUT,*) '-- Allocate control variable-',CLINE
    ALLOCATE(CTLVEC_STRUCT)
    IF (.NOT.ASSOCIATED(CVA_DATA)) CALL ABOR1('Call SUALCTV, but CVA_DATA is not set up')
    CALL SUALCTV(YDGEOMETRY,CTLVEC_STRUCT, CVA_DATA, JB_STRUCT)
  ELSE
    CALL ABOR1(' SU0YOMB: case where JB_STRUCT is used, but is not yet set up')
  ENDIF
ENDIF

!*    Initialize incremental 3D/4D-var
IF(.NOT.(ICONF == 7.OR.ICONF == 9)) THEN
  WRITE(NULOUT,*) '--- Set up incremental assimilation',CLINE
  CALL SURINC
ENDIF

!*    Initialize control of the Lanczos algorithm
LFORCEWR=.FALSE.
IF(.NOT.(ICONF == 0.OR.ICONF == 2.OR.NCONF == 302.OR.NCONF == 903)) THEN
  WRITE(NULOUT,*) '---- Set up Lanczos algorithm -------',CLINE
  LL_ALLOC_RLANBUF=(NCONF/100 == 6)
  IF(LL_ALLOC_RLANBUF) THEN
    CALL ALLOCATE_CTLVEC(YLTEMP)   
    ISIZEG=YLTEMP%NSIZEG
    CALL DEALLOCATE_CTLVEC(YLTEMP)   
  ELSE
    ISIZEG=0
  ENDIF
  CALL SULCZ(YDGEOMETRY,YDFIELDS,YDMTRAJ,LL_ALLOC_RLANBUF,ISIZEG,YDVARBC)
ENDIF

!*    Allocate the trajectory
IF(NCONF/100 == 1.OR.NCONF == 801.OR.NCONF == 601) THEN
  WRITE(NULOUT,*) '---- Allocate trajectory: SUALLT -------',CLINE
  CALL SUALLT(YDGEOMETRY,YDMTRAJ,YDFIELDS%YRGMV,YDFIELDS%YRSURF)
  LTRAJRESET=.TRUE.
ENDIF

IF (LTRAJHR .AND. LIFSTRAJ) THEN
  WRITE(NULOUT,*) '---- Allocate trajectory: ALLOCATE_TRAJECTORY -------',CLINE
  CALL ALLOCATE_TRAJECTORY(YDGEOMETRY,YDGMV,YDGMV5,YDSURF,JB_STRUCT)
ENDIF

!*    Get trajectory values for NOX LOG variable
IF(( NCHEM_ASSIM>0) .AND. NGRBNOXLOG> 0) THEN
  WRITE(NULOUT,*) '---- Get trajectory values for NOX LOG variable -------',CLINE
  IF (LIFSMIN.AND.(LTRAJHR.AND.LTRAJHR_ALTI) ) THEN

    ISTEP    = 0
    LLASTRAJ = .FALSE.
    CALL READ_TRAJECTORY(YDGEOMETRY,YDFIELDS,YDMTRAJ,ISTEP,LLASTRAJ,LREADGPTRAJ)

    IF (LREADGPTRAJ) THEN

      ALLOCATE (ZGMV5(NPROMA,NFLEVG,YT5%NDIM,NGPBLKS))
      ALLOCATE (ZGMV5S(NPROMA,YT5%NDIMS,NGPBLKS))
      ALLOCATE (ZGFL5(NPROMA,NFLEVG,NDIM5,NGPBLKS))

      ZGMV5=0._JPRB
      ZGMV5S=0._JPRB
      ZGFL5=0._JPRB

      CALL GET_TRAJ_GRID(YDGEOMETRY,TRAJEC(0),YDGMV,YDGMV5,ZGMV5,ZGMV5S,ZGFL5,1)

      DO JGFL=1,NDIM5
        IF (YCOMP(JGFL)%LTRAJIO) THEN
          YDGFL5%GFL5(:,:,YCOMP(JGFL)%MP,:)=ZGFL5(:,:,YCOMP(JGFL)%MP,:)
          DO JSTGLO=1,NGPTOT,NPROMA
            ICEND=MIN(NPROMA,NGPTOT-JSTGLO+1)
            IBL=(JSTGLO-1)/NPROMA+1
            YDGFL5%GFL5(1:ICEND,1:NFLEVG,YCOMP(JGFL)%MP,IBL)=ZGFL5(1:ICEND,1:NFLEVG,YCOMP(JGFL)%MP,IBL)
            YDGFL5%GFL5(ICEND+1:NPROMA,1:NFLEVG,YCOMP(JGFL)%MP,IBL)=0.0_JPRB
          ENDDO
        ENDIF
      ENDDO

      DEALLOCATE(ZGFL5)
      DEALLOCATE(ZGMV5)
      DEALLOCATE(ZGMV5S)

    ENDIF

  ENDIF
ENDIF

!*    Allocate Jb linearisation state
IF (NCONF==131 .OR. NCONF/100==5 .OR. NCONF/100==6 .OR. NCONF/100==8 .OR.&
  & (LTRAJHR .AND. LIFSTRAJ)) THEN
! Not needed for NCONF=401?
  WRITE(NULOUT,*) '---- Allocate Jb linearisation state -------',CLINE
  IF (.NOT.ASSOCIATED(BACKGROUND)) THEN
    CALL ABOR1 ('BACKGROUND HAS NOT BEEN ALLOCATED')
  ENDIF
  CALL ALLOCATE_JB_REF_STATE(YDGEOMETRY,BACKGROUND,YDGMV,YDGMV5,JB_STRUCT)
ENDIF

!*    Initialize Jb error covariance model
!*    Set up coefficients for humidity change of variable
IF (LBACKG .OR. LOBSC1) THEN
  WRITE(NULOUT,*) '---- Set up coefficients for humidity change of variable --',CLINE
  CALL SUJBCHVAR(YDVAB,YDDIMV,JB_STRUCT)
ENDIF

IF (LBACKG.OR.LSPBSBAL) THEN
  IF (LSPBSBAL.OR..NOT.JB_STRUCT%CONFIG%LJBENER) THEN
    WRITE(NULOUT,*) '---- Set up Jb balance operators --',CLINE
    IF(LELAM) THEN
      CALL SUEJBBAL(YDGEOMETRY,'STABAL96  ',JB_STRUCT)
    ELSE
      CALL SUJBBAL(YDGEOMETRY,'STABAL96  ',JB_STRUCT)
    ENDIF
  ENDIF
ENDIF

IF (LBACKG) THEN
  WRITE(NULOUT,*) '---- Set up Jb error covariances --',CLINE
  IF(LELAM) THEN
    IF (JB_STRUCT%WJBCONF%LJBWAVELET) THEN
      WRITE(NULOUT,*) 'Wavelet Jb in Aladin: good luck!'
      CALL SUEJBWAVELET_BMATRIX(YDDIMV)
    ELSE
      ! CALL SUEJBCOV(YDGEOMETRY,YDFIELDS,YDMTRAJ,YDVARBC,JB_STRUCT)
      CALL SUEJBCOV(YDGEOMETRY,YDFIELDS,YDMTRAJ,JB_STRUCT)
    ENDIF
  ELSE
    IF (JB_STRUCT%WJBCONF%LJBWAVELET) THEN
      
      IF ((.NOT.JB_STRUCT%WJBCONF%LJBWSTATS).OR.(JB_STRUCT%WJBCONF%LHYBRID_JB)) THEN
      ! sujbwavelet must be called before sujbwavstats if LHYBRID_JB=T
        CLFILE='wavelet.cv'
        CALL SUJBWAVELET(YDGEOMETRY,YDFIELDS,YDMTRAJ,JB_STRUCT,CLFILE)
        CALL SUJBWAVELET_STDEVS(YDGEOMETRY,JB_STRUCT)
      ENDIF

      IF (JB_STRUCT%WJBCONF%LJBWSTATS) THEN
        WRITE(NULOUT,*) '---- Calculate Wavelet Jb error covariances --',CLINE
        CALL SUJBWAVSTATS(YDGEOMETRY,YDFIELDS,YDMTRAJ,JB_STRUCT)

        IF(LBACKGERENORM)THEN
        ! Option to reload the matrix and to compute renormalisation coeffs of wavelet B
          WRITE(CLMAX,'(I3)') NSMAX
          CLFILE='wavelet_out_T'//TRIM(ADJUSTL(CLMAX))//'.cv'
          CALL SUJBWAVELET(YDGEOMETRY,YDFIELDS,YDMTRAJ,JB_STRUCT,CLFILE)
          CALL SUJBWAVELET_STDEVS(YDGEOMETRY,JB_STRUCT)
          WRITE(NULOUT,*) '---- Variational job: Compute renormalisation coefficient',&
         & 'for the variance -------------------'
          CALL SUJBWAVRENORM(YDGEOMETRY,YDFIELDS,YDMTRAJ,JB_STRUCT)
        ENDIF

        ! The statistics file has been written and closed.
        ! Normal exit
        WRITE (NULOUT,*) 'Jb stats file has been written'
        WRITE (NULOUT,*) 'Normal exit'
        CALL IOSTREAM_STATS
        CALL GSTATS(0,1)
        CALL GSTATS_OUTPUT_IFS
        IF (NPROC > 1) THEN
          CALL MPL_BARRIER(CDSTRING='SU0YOMB')  
        ENDIF
        CALL MPL_END()
        STOP

      ENDIF

    ELSE
      IF (.NOT.ASSOCIATED(BACKGROUND)) THEN
        CALL ABOR1 ('BACKGROUND HAS NOT BEEN ALLOCATED')
      ENDIF
      CALL SUJBCOV(YDGEOMETRY,YDFIELDS,YDMTRAJ,JB_STRUCT)
      IF (NCONF==801.AND. .NOT.LBACKG) THEN
        CALL SUECGES(YDGEOMETRY,YDFIELDS,YDMTRAJ,BACKGROUND,YDVARBC,JB_STRUCT)
      ENDIF
      !     Test the correctness of Jb code (activate by hand)
      IF (JB_STRUCT%CONFIG%LJBTEST) THEN
        IF(.NOT.ASSOCIATED(CVA_STRUCT)) CALL ABOR1(' SU0YOMB: case where CVA_STRUCT is used, but is not yet set up')
        CALL SUJBTEST(YDGEOMETRY,YDMTRAJ,YDFIELDS%YRGFL,YDFIELDS%YRGMV,&
                    & JB_STRUCT%CONFIG%NJBVERB,CVA_STRUCT%YVAZX,YDVARBC)
      ENDIF
    ENDIF
  ENDIF
  
  IF (LELAM) THEN
    IF (LEJK) THEN
      WRITE(NULOUT,*) '---- Set up Jk error covariances --',CLINE
      CALL SUELLJK(YDGEOMETRY,NULOUT)
      CALL SUEJKNORM(YDDIMV,JB_STRUCT)
      WRITE(NULOUT,*) '---- Set up Jk modulation wrt level and param --',CLINE
      CALL SUEMODJK(YDGEOMETRY%YRSTA,YDDIMV)
    ENDIF
  ENDIF

  IF(.NOT.(ICONF == 8) .AND..NOT.(ICONF == 6)) THEN
    WRITE(NULOUT,*) '---- Set up Background fields --',CLINE
    IF (.NOT.ASSOCIATED(BACKGROUND)) THEN
      CALL ABOR1 ('BACKGROUND HAS NOT BEEN ALLOCATED')
    ENDIF
    CALL SUECGES(YDGEOMETRY,YDFIELDS,YDMTRAJ,BACKGROUND,YDVARBC,JB_STRUCT)

     IF(.NOT.LELAM .OR. .NOT.JB_STRUCT%CONFIG%LSPFCE) THEN
       WRITE(NULOUT,*) '---- Set up Jb gridpoint background error stdev --',CLINE
       CALL SUINFCE(YDGEOMETRY,YDFIELDS%YRGMV,YDMTRAJ%YRGMV5,YDFIELDS%YRSURF,BACKGROUND,JB_STRUCT)
     ENDIF
  ENDIF

!*    Initialize Jq error covariances
  IF (LMODERR) THEN
    WRITE(NULOUT,*) '---- Set up Jq error covariances --',CLINE
    CALL SUJQ(YDGEOMETRY)
  ENDIF
  IF (LCVALPHA) THEN
    WRITE(NULOUT,*) '---- Set up Jalpha error covariances --',CLINE
    CALL SETUP_HYBENS(NULBDESH,CVA_DATA)
    CALL SUJACOS(YDGEOMETRY,CVA_DATA)
    CALL SUALPFLD(YDGEOMETRY,CTLVEC_STRUCT,CVA_DATA)
    CALL ALLOCATE_ENS(YDGEOMETRY,JB_STRUCT)
    CALL FLUSH(NULOUT) 
    IF (CVA_DATA%NALPHA>0) & 
     & CALL GET_HYBRID_ENS(YDGEOMETRY,YDGMV,YDGMV5,YDGFL5,BACKGROUND,JB_STRUCT)
  ENDIF 
ENDIF

!*    Initialize scalar product
IF(.NOT.(NCONF == 1.OR.NCONF == 302.OR.ICONF == 2.OR.ICONF == 9.OR.ICONF == 7 )) THEN
  WRITE(NULOUT,*) '---- Set up scalar product --------',CLINE
  IF (LELAM) THEN
    ALLOCATE (SCALP_STRUCT)
    CALL SUESCAL(YDGEOMETRY,YDFIELDS,YDMTRAJ,SCALP_STRUCT,YDVARBC,JB_STRUCT)
  ELSE
    ALLOCATE (SCALP_STRUCT)
    CALL SUSCAL(YDGEOMETRY,YDFIELDS,YDMTRAJ,SCALP_STRUCT,YDVARBC,JB_STRUCT)
  ENDIF
ENDIF

!*    Initialize subspace
IF(.NOT.(ICONF == 0.OR.ICONF == 2.OR.NCONF == 302).AND.L_SUBSPACE_SVS) THEN
  WRITE(NULOUT,*) '---- Set up subspace for SV computation -------',CLINE
  CALL SU_SUBSPACE(YDGEOMETRY)
ENDIF

!     Read in forecast errors
IF (LSCREEN .AND. .NOT. LMONITOR_OBSTATFC) THEN
  WRITE(NULOUT,*) '---- Read in forecast errors ---',CLINE
  CALL INIFGER
ENDIF

WRITE(NULOUT,*) 'NT: LWINDFARM = ', LWINDFARM

!    If required allocate empty skeleton of trajectory structure
IF (.NOT. LTRAJALLOC .AND. ((NCONF /= 401).AND.(NCONF /= 501).AND. (&
                          & NCONF /= 601).AND.(NCONF /= 801) )) THEN 
  CALL ALLOCATE_EMPTY_TRAJECTORY(YDDIM)
  ! ltrajalloc=.true.
ENDIF
!     Set up Stochastic Physics
WRITE(NULOUT,'(A72)') '--- Set up stochastic physics, SPBS, CABS '//CLINE
CALL INI_CUCONVCA(YDGEOMETRY)
CALL SURAND1(YDGEOMETRY)
!     Set up spectral stochastic diabatic tendencies
WRITE(NULOUT,'(A72)') '--- Set up stochastically perturbed parametrization tendencies '//CLINE
WRITE(NULOUT,*)       '      SPPT a.k.a. stochastic physics with spectral pattern'
CALL SUSPSDT(YDGEOMETRY)
IF (LSPSDT.AND.LSTOPH) THEN
  CALL ABOR1('LSPSDT.AND.LSTOPH==T is not allowed!')
ENDIF


!*   Initialize wind farm -- this may be moved to su0yomb
IF (LWINDFARM) THEN
    WRITE(NULOUT,*) '------ NT: Set up wind farms ',CLINE
    CALL SUWFP(YDGEOMETRY)
ENDIF

!     Set up stochastically perturbed parameterisation scheme

WRITE(NULOUT,*) '--- Set up stochastically perturbed parametrization scheme (SPP)  ',CLINE
CALL GET_SPP_CONF(YSPP_CONFIG)
CALL INI_SPP(YDGEOMETRY,YRRIP,YSPP_CONFIG, YSPP)

!*    Create io server file templates
!     provide optional GEOMETRY argument here, since these are MODEL processes rather than IO_SERV ones
  WRITE(NULOUT,*) '---- Create io server file templates --',CLINE
  CALL IO_SERV_SUIOSCTMPL(IO_SERV_C001, YDGEOMETRY)


! Set up gravity wave diagnostics
CALL SETUP_GWDIAG(YDDIM)
!    Set up flexible physics-dynamics interface
WRITE(NULOUT,*) '---- Set up flexible physics-dynamics interface --',CLINE
CALL SUINTFLEX

IF (FP_SERV_C001%LFP_CLIENT .OR. FP_SERV_C001%LFP_SERVER) THEN
  CALL FP_SERV_SUIOSCTMPL (FP_SERV_C001, YDGEOMETRY)
  IF (.NOT. LELAM) CALL FP_SERV_CPFPFILTER (FP_SERV_C001, YDGEOMETRY)
ENDIF

WRITE(NULOUT,*) '-------------------------------------',CLINE
WRITE(NULOUT,*) '------ END OF SETUPS at level 0 -----',CLINE
WRITE(NULOUT,*) '-------------------------------------',CLINE
WRITE(NULOUT,*) ' '
WRITE(NULOUT,*) ' '
CALL FLUSH(NULOUT)
CALL GSTATS(39,1)

!     ------------------------------------------------------------------

END ASSOCIATE
END ASSOCIATE
IF (LHOOK) CALL DR_HOOK('SU0YOMB',1,ZHOOK_HANDLE)
END SUBROUTINE SU0YOMB


