SUBROUTINE SUWFP_RC

!
! Read the `wind_turbine_coordinates.tab` table containing
! the lat/lon coordinates + type of each individual wind turbine
!
USE PARKIND1,     ONLY : JPIM,JPRB
USE YOMLUN,       ONLY : NULOUT
USE YOMCST,       ONLY : RPI
USE MPL_MODULE,   ONLY : MPL_MYRANK, MPL_BROADCAST
USE WINDFARM_MOD, ONLY : N_WT_GLOB, WT_LAT_GLOB, WT_LON_GLOB, & 
                    & MWT_TYPE_GLOB, WTT_HUBHEIGHT_GLOB, WTT_RADIUS_GLOB


IMPLICIT NONE

INTEGER(KIND=JPIM) :: IFILE_UNIT = 666
INTEGER(KIND=JPIM) :: I_WT, MPIID, IERR=0
LOGICAL            :: LFILE_EXISTS
CHARACTER(28)      :: CFILENAME = 'wind_turbine_coordinates.tab'
CHARACTER(1)       :: CDUMMY

#include "suwfp_sk.intfb.h"

! Only MPI rank=1 reads the data, and broadcasts it
! to the other MPI tasks

MPIID = MPL_MYRANK()
IF (MPIID == 1) THEN

    INQUIRE(FILE=CFILENAME, EXIST=LFILE_EXISTS)

    IF (LFILE_EXISTS) THEN
        WRITE(NULOUT,*) 'Processing ', CFILENAME

        ! Open file, and skip the comments (#)
        OPEN (IFILE_UNIT, FILE=CFILENAME)
        CALL SUWFP_SK(IFILE_UNIT)

        ! COUNT NUMBER OF TURBINES
        N_WT_GLOB = 0
        DO WHILE (IERR == 0)
            READ(IFILE_UNIT, *, IOSTAT=IERR) CDUMMY
            IF (IERR == 0) N_WT_GLOB = N_WT_GLOB + 1
        END DO

        WRITE(NULOUT,*) 'Found ', N_WT_GLOB, ' wind turbines'
    ELSE
        WRITE(NULOUT,*) 'File ', CFILENAME, ' missing'
        CALL ABOR1('Routine READ_TURBINE_COORDINATES failed')
    ENDIF
ENDIF

! Broadcast global wind turbine count to all tasks
CALL MPL_BROADCAST(N_WT_GLOB, KTAG=0, CDSTRING='WT_COUNT')

ALLOCATE( WT_LAT_GLOB(N_WT_GLOB), WT_LON_GLOB(N_WT_GLOB), MWT_TYPE_GLOB(N_WT_GLOB), &
        & WTT_RADIUS_GLOB(N_WT_GLOB), WTT_HUBHEIGHT_GLOB(N_WT_GLOB) )

! Read the wind turbine locations
IF (MPIID == 1) THEN

    ! Read the turbine coordinates and types
    REWIND(IFILE_UNIT)
    CALL SUWFP_SK(IFILE_UNIT)

    DO I_WT=1, N_WT_GLOB
        READ(IFILE_UNIT, *, IOSTAT=IERR) WT_LON_GLOB(I_WT), WT_LAT_GLOB(I_WT), MWT_TYPE_GLOB(I_WT),WTT_RADIUS_GLOB(I_WT), WTT_HUBHEIGHT_GLOB(I_WT)
   
        ! check hub height and radius are reasonable
        IF ((WTT_RADIUS_GLOB(I_WT)<1._JPRB).OR.(WTT_RADIUS_GLOB(I_WT)>100._JPRB)) THEN
            WRITE(NULOUT,*) 'SOMETHING WRONG RADIUS', WTT_RADIUS_GLOB(I_WT),I_WT
            CALL ABOR1('Routine READ_TURBINE_COORDINATES failed because of wrong radius')
        ENDIF
        IF ((WTT_HUBHEIGHT_GLOB(I_WT)<10._JPRB).OR.(WTT_HUBHEIGHT_GLOB(I_WT)>300._JPRB)) THEN
            WRITE(NULOUT,*) 'SOMETHING WRONG HUBHEIGHT', WTT_HUBHEIGHT_GLOB(I_WT),I_WT
            CALL ABOR1('Routine READ_TURBINE_COORDINATES failed because of wrong hubheight')
        ENDIF
    END DO

    ! Degrees in radians
    WT_LAT_GLOB(:) = WT_LAT_GLOB(:) * RPI/180._JPRB
    WT_LON_GLOB(:) = WT_LON_GLOB(:) * RPI/180._JPRB

    CLOSE(IFILE_UNIT)

END IF
! Broadcast to other MPI tasks
CALL MPL_BROADCAST(WT_LON_GLOB,       KTAG=0, CDSTRING='WINDLON')
CALL MPL_BROADCAST(WT_LAT_GLOB,       KTAG=1, CDSTRING='WINDLAT')
CALL MPL_BROADCAST(MWT_TYPE_GLOB,     KTAG=2, CDSTRING='WINDTYPE')
CALL MPL_BROADCAST(WTT_RADIUS_GLOB,   KTAG=3, CDSTRING='WINDRADIUS')
CALL MPL_BROADCAST(WTT_HUBHEIGHT_GLOB, KTAG=4, CDSTRING='WINDHHEIGHT')

END SUBROUTINE SUWFP_RC
